@@ -1,8 +1,8 @@
 /*
- * NASPRO - The NASPRO Architecture for Sound Processing
+ * NASPRO - The NASPRO Architecture for Sound PROcessing
  * A simple command line effect processor using LV2 plugins
  *
- * Copyright (C) 2011, 2012 LV2proc development team
+ * Copyright (C) 2011-2013 LV2proc development team
  *
  * See the COPYING file for license conditions.
  */
@@ -54,9 +54,9 @@ strtoul_nl(unsigned long *v, const char *s)
 	err = errno;
 
 	setlocale(LC_NUMERIC, locale);
-	
+
 	errno = (err == 0) ? errsv : err;
-	
+
 	return (err == 0) && (*c == '\0');
 }
 
@@ -76,9 +76,9 @@ strtof_nl(float *v, const char *s)
 	err = errno;
 
 	setlocale(LC_NUMERIC, locale);
-	
+
 	errno = (err == 0) ? errsv : err;
-	
+
 	return (err == 0) && (*c == '\0');
 }
 
@@ -193,7 +193,7 @@ cmd_parse(int argc, char *argv[])
 		if (!strcmp(argv[i], "--version"))
 		  {
 			printf(PACKAGE " (" PACKAGE_NAME ") " VERSION "\n");
-			printf("Copyright (C) 2011, 2012 LV2proc development "
+			printf("Copyright (C) 2011-2013 LV2proc development "
 			       "team\n");
 			printf("This is free software; see the source for "
 			       "copying conditions.  There is NO\n");
@@ -406,7 +406,8 @@ cmd_fini()
 static LilvWorld *lv2_world;
 static const LilvPlugins *lv2_plugins;
 static LilvNode *lv2_input_node, *lv2_output_node, *lv2_audio_node,
-		*lv2_ctrl_node, *lv2_conn_opt_node, *lv2_rep_lat_node;
+		*lv2_ctrl_node, *lv2_conn_opt_node, *lv2_rep_lat_node,
+		*lv2_sample_rate_node;
 static const LilvPort *lv2_latency_port = NULL;
 
 static uint32_t audio_in_min = 0, audio_in_max = 0, audio_in_n, audio_out_n = 0,
@@ -434,7 +435,9 @@ lv2_init()
 					 LILV_NS_LV2 "connectionOptional");
 	lv2_rep_lat_node = lilv_new_uri(lv2_world,
 					LILV_NS_LV2 "reportsLatency");
-	
+	lv2_sample_rate_node = lilv_new_uri(lv2_world,
+					    LILV_NS_LV2 "sampleRate");
+
 	return 1;
 }
 
@@ -597,6 +600,7 @@ lv2_plugin_inspect_ports(const LilvPlugin *plugin)
 static void
 lv2_fini()
 {
+	lilv_node_free(lv2_sample_rate_node);
 	lilv_node_free(lv2_rep_lat_node);
 	lilv_node_free(lv2_conn_opt_node);
 	lilv_node_free(lv2_ctrl_node);
@@ -692,7 +696,7 @@ proc_bufs_alloc(sf_count_t frames)
 	  }
 
 	return true;
-	
+
 norm_err:
 	if (ctrl_out_values != NULL)
 		free(ctrl_out_values);
@@ -716,7 +720,8 @@ in_err:
 }
 
 static void
-proc_bufs_connect(const LilvPlugin *plugin, LilvInstance *instance)
+proc_bufs_connect(const LilvPlugin *plugin, LilvInstance *instance,
+		  double sample_rate)
 {
 	uint32_t audio_in_i = 0, audio_out_i = 0, ctrl_in_i = 0, ctrl_out_i = 0;
 
@@ -724,13 +729,13 @@ proc_bufs_connect(const LilvPlugin *plugin, LilvInstance *instance)
 	for (uint32_t i = 0; i < n; i++)
 	  {
 		const LilvPort *port = lilv_plugin_get_port_by_index(plugin, i);
-		
+
 		if (lilv_port_is_a(plugin, port, lv2_audio_node))
 		  {
 			if (lilv_port_has_property(plugin, port,
 						   lv2_conn_opt_node))
 				continue;
-			
+
 			if (lilv_port_is_a(plugin, port, lv2_input_node))
 			  {
 				lilv_instance_connect_port(instance, i,
@@ -760,45 +765,59 @@ proc_bufs_connect(const LilvPlugin *plugin, LilvInstance *instance)
 			  {
 				if ((min != NULL) && (max != NULL))
 				  {
-					if ((conn->value
-					     < lilv_node_as_float(min))
-					    || (conn->value
-					        > lilv_node_as_float(max)))
+					double amin = lilv_node_as_float(min);
+					double amax = lilv_node_as_float(max);
+
+					if (lilv_port_has_property(plugin, port,
+						lv2_sample_rate_node))
+					  {
+						amin *= sample_rate;
+						amax *= sample_rate;
+					  }
+
+					if ((conn->value < amin)
+					    || (conn->value > amax))
 						PRINTF_WARNING("out of range "
 							"value for port '%s' "
 							"(min: %g, max: %g)",
 							lilv_node_as_string(
 							  lilv_port_get_symbol(
 							    plugin, port)),
-							lilv_node_as_float(min),
-							lilv_node_as_float(max)
-							);
+							amin, amax);
 				  }
 				else if (min != NULL)
 				  {
-					if (conn->value
-					     < lilv_node_as_float(min))
+					double amin = lilv_node_as_float(min);
+
+					if (lilv_port_has_property(plugin, port,
+						lv2_sample_rate_node))
+						amin *= sample_rate;
+
+					if (conn->value < amin)
 						PRINTF_WARNING("out of range "
 							"value for port '%s' "
 							"(min: %g)",
 							lilv_node_as_string(
 							  lilv_port_get_symbol(
 							    plugin, port)),
-							lilv_node_as_float(min)
-							);
+							amin);
 				  }
 				else if (max != NULL)
 				  {
-					if (conn->value
-					     > lilv_node_as_float(max))
+					double amax = lilv_node_as_float(max);
+
+					if (lilv_port_has_property(plugin, port,
+						lv2_sample_rate_node))
+						amax *= sample_rate;
+
+					if (conn->value > amax)
 						PRINTF_WARNING("out of range "
 							"value for port '%s' "
 							"(max: %g)",
 							lilv_node_as_string(
 							  lilv_port_get_symbol(
 							    plugin, port)),
-							lilv_node_as_float(max)
-							);
+							amax);
 				  }
 
 				lilv_instance_connect_port(instance, i,
@@ -853,7 +872,7 @@ proc_bufs_connect(const LilvPlugin *plugin, LilvInstance *instance)
 	for (uint32_t i = 0; (i < n) && (audio_in_i < audio_in_n); i++)
 	  {
 		const LilvPort *port = lilv_plugin_get_port_by_index(plugin, i);
-		
+
 		if (!lilv_port_has_property(plugin, port, lv2_conn_opt_node))
 			continue;
 
@@ -976,7 +995,7 @@ proc_run(SNDFILE *in, SNDFILE *out, LilvInstance *instance)
 			pre_time = clock();
 
 		lilv_instance_run(instance, frames);
-		
+
 		if (timing_do)
 			process_time += clock() - pre_time;
 
@@ -1219,8 +1238,8 @@ main(int argc, char *argv[])
 		goto out_open_err;
 	  }
 
-	proc_bufs_connect(plugin, instance);
-	
+	proc_bufs_connect(plugin, instance, in_info.samplerate);
+
 	lilv_instance_activate(instance);
 
 	proc_run(in, out, instance);
@@ -1249,6 +1268,6 @@ lv2_plugin_err:
 
 lv2_init_err:
 	cmd_fini();
-	
+
 	return exit_code;
 }
